class Slider
{
  PVector _p1; // starting point
  PVector _p2; // end point

  int     _h; // height of slider
  int     _w; // width of slider
  float   _pos; // position slider
  boolean    _dir; // direction 'horizontal'|'vertical'

  // x/y of slider, width/height of slider box, position knob, direction true:'horizontal' | false:'vertical'
  Slider(int x,int y,int w,int h,float pos, boolean dir)
  {
    _dir = dir;

    if (_dir==true) { // 'horizontal'
      _p1 = new PVector(x,y + h *.5);
      _p2 = new PVector(x+w,y + h *.5);
    } 
    else {
      _p1 = new PVector(x+w*.5, y);
      _p2 = new PVector(x+w*.5, y+h);
    }
    _h = h;
    _w = w;

    _pos = pos; // x value for horizontal, y value for horizontal
    if(_pos > 1.0)
      _pos = 1.0;
    else if(_pos < 0.0)
      _pos = 0.0;
      
    smooth(); // svg loaded for slider styling
  }

  float pos()
  {
    return _pos;
  }

  boolean isHit(int x,int y)
  {
    if (_dir==true && x > _p1.x && x < _p2.x &&
      y > _p1.y - 10 && y < _p1.y + 10) { // horizontal
      return true;
    } 
    else if (_dir==false && x > _p1.x - 10 && x < _p1.x + 10 &&
      y > _p1.y && y < _p2.y) { // vertical
      return true;
    } 
    else {
      return false;
    }
  }

  void draw()
  {
    pushStyle();
      drawSlider();
      drawKnob(false);
    popStyle();
  }

  void drawSlider()
  {
    PShape sliderBg = loadShape("slider_bg.svg");
    PShape sliderEnd = loadShape("slider_end.svg");

    if (_dir==true) { // horizontal
      shape(sliderEnd,_p1.x-10/1.5,_p1.y-_h/1.5*.5,20/1.5,20/1.5); // end left
      shape(sliderEnd,_p2.x-10/1.5,_p1.y-_h/1.5*.5,20/1.5,20/1.5); // end right
      shape(sliderBg,_p1.x,_p1.y-_h/1.5*.5,_w,_h/1.5); // strip
    } 
    else {
      sliderBg.rotate(radians(-90));
      sliderEnd.rotate(radians(-90));
      shape(sliderEnd,_p1.x-_w/1.5*.5,_p1.y+10/1.5,20/1.5,20/1.5); // end left
      shape(sliderEnd,_p1.x-_w/1.5*.5,_p2.y+10/1.5,20/1.5,20/1.5); // end right
      shape(sliderBg,_p1.x-_w/1.5*.5,_p2.y,_w/1.5,_h); // strip
    }
  }

  void drawKnob(boolean hover)
  {
    PShape sliderKnob = null;
    if (hover==false) {
      sliderKnob = loadShape("slider_knob.svg");
    } else {
      sliderKnob = loadShape("slider_knob_hover.svg");
    }
    
    if (_dir==true) { // horizontal
      PVector dir = PVector.sub(_p2,_p1);
      PVector pos = PVector.add( _p1, PVector.mult(dir,_pos));
      shape(sliderKnob,pos.x-10, pos.y - _h*.5,20,20);
    } 
    else {
      PVector dir = PVector.sub(_p1,_p2);
      PVector pos = PVector.add( _p2, PVector.mult(dir,_pos));
      shape(sliderKnob,pos.x-_w*.5, pos.y - 10,20,20);
    }
  }

  void mousePos(int x,int y,boolean pressed)
  {
    if(pressed)
    {
      if(isHit(x,y) && _dir==true)
      {  // move knob horizontal
        PVector dir = PVector.sub(_p2,_p1);
        _pos = 1.0 / dir.mag() * (x - _p1.x);
      }
      if(isHit(x,y) && _dir==false)
      {  // move knob vertical
        PVector dir = PVector.sub(_p1,_p2);
        _pos = 1.0 / dir.mag() * (_p2.y - y);
      }
    }
    else
    {  // mouse over: highlight knob
      if(isHit(x,y)) drawKnob(true);
    }
  }
}

